@@ -71,9 +71,7 @@ extern char*    cvtItoa(const IntRep* x, char* fmt, int& fmtlen, int base,
                         char fillchar, char Xcase, int showpos);
 extern IntRep*  atoIntRep(const char* s, int base = 10);
 extern long     Itolong(const IntRep*);
-extern double   Itodouble(const IntRep*);
 extern int      Iislong(const IntRep*);
-extern int      Iisdouble(const IntRep*);
 extern long     lg(const IntRep*);
 
 extern IntRep _ZeroRep, _OneRep, _MinusOneRep;
@@ -91,45 +89,44 @@ public:
                   Integer(const Integer&);
 
                   ~Integer();
-
-  void            operator =  (const Integer&);
-  void            operator =  (long);
+  Integer&        operator =  (const Integer&);
+  Integer&        operator =  (long);
 
 // unary operations to self
 
-  void            operator ++ ();
-  void            operator -- ();
+  Integer&        operator ++ ();
+  Integer&        operator -- ();
   void            negate();          // negate in-place
   void            abs();             // absolute-value in-place
   void            complement();      // bitwise complement in-place
 
 // assignment-based operations
 
-  void            operator += (const Integer&);
-  void            operator -= (const Integer&);
-  void            operator *= (const Integer&);
-  void            operator /= (const Integer&);
-  void            operator %= (const Integer&);
-  void            operator <<=(const Integer&);
-  void            operator >>=(const Integer&);
-  void            operator &= (const Integer&);
-  void            operator |= (const Integer&);
-  void            operator ^= (const Integer&);
+  Integer&        operator += (const Integer&);
+  Integer&        operator -= (const Integer&);
+  Integer&        operator *= (const Integer&);
+  Integer&        operator /= (const Integer&);
+  Integer&        operator %= (const Integer&);
+  Integer&        operator <<=(const Integer&);
+  Integer&        operator >>=(const Integer&);
+  Integer&        operator &= (const Integer&);
+  Integer&        operator |= (const Integer&);
+  Integer&        operator ^= (const Integer&);
 
-  void            operator += (long);
-  void            operator -= (long);
-  void            operator *= (long);
-  void            operator /= (long);
-  void            operator %= (long);
-  void            operator <<=(long);
-  void            operator >>=(long);
-  void            operator &= (long);
-  void            operator |= (long);
-  void            operator ^= (long);
+  Integer&        operator += (long);
+  Integer&        operator -= (long);
+  Integer&        operator *= (long);
+  Integer&        operator /= (long);
+  Integer&        operator %= (long);
+  Integer&        operator <<=(long);
+  Integer&        operator >>=(long);
+  Integer&        operator &= (long);
+  Integer&        operator |= (long);
+  Integer&        operator ^= (long);
 
 // (constructive binary operations are inlined below)
 
-#ifdef __GNUG__
+#if defined (__GNUG__) && ! defined (__STRICT_ANSI__)
   friend Integer operator <? (const Integer& x, const Integer& y); // min
   friend Integer operator >? (const Integer& x, const Integer& y); // max
 #endif
@@ -198,15 +195,10 @@ public:
 // coercion & conversion
 
   int             fits_in_long() const { return Iislong(rep); }
-  int             fits_in_double() const { return Iisdouble(rep); }
+  int             fits_in_double() const;
 
-#if 0
-  // There two operators cause a number of ambiguities.
-                  operator long() const { return Itolong(rep); }
-                  operator double() const { return Itodouble(rep); }
-#endif
   long		  as_long() const { return Itolong(rep); }
-  double	  as_double() const { return Itodouble(rep); }
+  double	  as_double() const;
 
   friend char*    Itoa(const Integer& x, int base = 10, int width = 0);
   friend Integer  atoI(const char* s, int base = 10);
@@ -297,14 +289,16 @@ inline Integer::Integer(const Integer&  y) :rep(Icopy(0, y.rep)) {}
 
 inline Integer::~Integer() { if (rep && !STATIC_IntRep(rep)) delete rep; }
 
-inline void  Integer::operator = (const Integer&  y)
+inline Integer&  Integer::operator = (const Integer&  y)
 {
   rep = Icopy(rep, y.rep);
+  return *this;
 }
 
-inline void Integer::operator = (long y)
+inline Integer& Integer::operator = (long y)
 {
-  rep = Icopy_long(rep, y); 
+  rep = Icopy_long(rep, y);
+  return *this;
 }
 
 inline int Integer::initialized() const
@@ -563,116 +557,136 @@ inline int operator >= (const Integer&  x, long y)
 }
 
 
-inline void  Integer::operator += (const Integer& y)
+inline Integer&  Integer::operator += (const Integer& y)
 {
   add(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator += (long y)
+inline Integer&  Integer::operator += (long y)
 {
   add(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator ++ ()
+inline Integer& Integer::operator ++ ()
 {
   add(*this, 1, *this);
+  return *this;
 }
 
 
-inline void  Integer::operator -= (const Integer& y)
+inline Integer& Integer::operator -= (const Integer& y)
 {
   sub(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator -= (long y)
+inline Integer& Integer::operator -= (long y)
 {
   sub(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator -- ()
+inline Integer& Integer::operator -- ()
 {
   add(*this, -1, *this);
+  return *this;
 }
 
 
 
-inline void Integer::operator *= (const Integer& y)
+inline Integer& Integer::operator *= (const Integer& y)
 {
   mul(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator *= (long y)
+inline Integer& Integer::operator *= (long y)
 {
   mul(*this, y, *this);
+  return *this;
 }
 
 
-inline void  Integer::operator &= (const Integer& y)
+inline Integer& Integer::operator &= (const Integer& y)
 {
   and(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator &= (long y)
+inline Integer& Integer::operator &= (long y)
 {
   and(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator |= (const Integer& y)
+inline Integer& Integer::operator |= (const Integer& y)
 {
   or(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator |= (long y)
+inline Integer& Integer::operator |= (long y)
 {
   or(*this, y, *this);
+  return *this;
 }
 
 
-inline void  Integer::operator ^= (const Integer& y)
+inline Integer& Integer::operator ^= (const Integer& y)
 {
   xor(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator ^= (long y)
+inline Integer& Integer::operator ^= (long y)
 {
   xor(*this, y, *this);
+  return *this;
 }
 
 
 
-inline void Integer::operator /= (const Integer& y)
+inline Integer& Integer::operator /= (const Integer& y)
 {
   div(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator /= (long y)
+inline Integer& Integer::operator /= (long y)
 {
   div(*this, y, *this);
+  return *this;
 }
 
 
-inline void Integer::operator <<= (const Integer&  y)
+inline Integer& Integer::operator <<= (const Integer&  y)
 {
   lshift(*this, y, *this);
+  return *this;
 }
 
-inline void Integer::operator <<= (long  y)
+inline Integer& Integer::operator <<= (long  y)
 {
   lshift(*this, y, *this);
+  return *this;
 }
 
 
-inline void Integer::operator >>= (const Integer&  y)
+inline Integer& Integer::operator >>= (const Integer&  y)
 {
   rshift(*this, y, *this);
+  return *this;
 }
 
-inline void  Integer::operator >>= (long y)
+inline Integer& Integer::operator >>= (long y)
 {
   rshift(*this, y, *this);
+  return *this;
 }
 
-#ifdef __GNUG__
+#if defined (__GNUG__) && ! defined (__STRICT_ANSI__)
 inline Integer operator <? (const Integer& x, const Integer& y)
 {
   return (compare(x.rep, y.rep) <= 0) ? x : y;
@@ -731,7 +745,7 @@ inline long lg(const Integer& x)
 
 // constructive operations 
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 inline Integer  operator +  (const Integer& x, const Integer& y) return r
 {
@@ -1091,13 +1105,15 @@ inline Integer  gcd(const Integer& x, const Integer& y)
 
 #endif  /* NO_NRV */
 
-inline void Integer::operator %= (const Integer& y)
+inline Integer& Integer::operator %= (const Integer& y)
 {
   *this = *this % y; // mod(*this, y, *this) doesn't work.
+  return *this;
 }
 
-inline void Integer::operator %= (long y)
+inline Integer& Integer::operator %= (long y)
 {
   *this = *this % y; // mod(*this, y, *this) doesn't work.
+  return *this;
 }
 #endif /* !_Integer_h */
